#!/usr/bin/env python
#
# Copyright (C) 2014 - Renzo Davoli
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# 

import pygame, sys, copy, json, os
import argparse
import subprocess
from pygame.locals import *
xclipboard=1

inch=2.54

class mille:
	'''
	pcb object: size in mm, width/height in .1" elements.
	table: the same integer represents the same track
	drill: boolean (in the future values can represent different diameters)
	rear: for two sided pcb, not yet implemented
	'''
	def __init__(self):
		self.size=(0,0)
		self.width,self.height=0,0
		self.table=[]
		self.rear=[]
		self.drill=[]
	def valid(self,i,j):
		return True if i>0 and j>0 and i<self.width+1 and j<self.height+1 else False
	def j(self,j):
		return j-(self.height+1) if j > self.height else j
	def new(self,size=(120,100),twosided=0):
		self.width,self.height=size
		self.size=size
		self.width,self.height=int(self.width//inch),int(self.height//inch)
		self.drill=[[0 for _ in range(self.width+2)] for _ in range(self.height+2)]
		self.table=[[1 if self.valid(i,j) else 0 for i in range(self.width+2)] 
			for j in range(self.height+2)]
		if twosided:
			self.rear=[[1 if self.valid(i,j) else 0 for i in range(self.width+2)] 
				for j in range(self.height+2)]

xoff=100
WHITE=(255,255,255)
GRAY=(230,230,230)
GREEN=(100,230,100)
LGREEN=(200,255,200)
RED=(230,100,100)
HIGHLIGHT=(100,230,230)
BLACK=(0,0,0)
EDIT,SELECT=0,1
mode=EDIT
undostack=[]
currentselection=None
class clipboard:
	table=[]
	drill=[]
	rear=[]

def winsize(pcb,twosided):
	return pcb.width*10+20+xoff,pcb.height*10+20+((pcb.height*10+10) if twosided else 0)

class defargs:
	size="120x100"

def loadrc():
	try:
		with open(os.path.expanduser("~/.milleforirc"), "r") as f:
			loaded=json.load(f)
			for f in loaded:
				defargs.__dict__[f]=loaded[f]
	except IOError:
		pass

def main():
	pcb=mille()
	pcb.filename=""
	loadrc()
	global display

	def sizetype(s):
		spl=s.split('x')
		if len(spl) != 2:
			msg="size syntax: width x height in mm. Es: 120x100"
			raise argparse.ArgumentTypeError(msg)
		x,y=int(spl[0]),int(spl[1])
		if x == 0 or y == 0:
			msg="size: incorrect value"
			raise argparse.ArgumentTypeError(msg)
		return x,y

	parser = argparse.ArgumentParser(description='Drawing program for simple PCBs.')
	parser.add_argument('path', metavar='PATH', help='pathname')
	parser.add_argument('--size', '-s', dest='size', type=sizetype, help='size (in mm) default:{}'.format(defargs.size))
	parser.add_argument('--twosided', '-2', action="store_true", help='two sided PCB')
	parser.add_argument('--noxflip', action="store_true", help='backside: do not flip x coords')
	parser.add_argument('--xyflip', action="store_true", help='backside: flip x and y')
	parser.add_argument('--centered', action="store_true", help='xy centered')
	parser.set_defaults(size=sizetype(defargs.size))
	
	args = parser.parse_args()
	if 'twosided' not in args.__dict__:
		args.twosided=False
	
	pcb.filename=args.path
	pygame.init()
	if file_load(pcb):
		args.twosided = True if pcb.rear else False
	else:
		pcb.new(args.size,args.twosided)
	
	if args.centered:
		xsize,ysize=pcb.size
		args.posxoff=(xsize - pcb.width * inch)/2
		args.posyoff=(ysize - pcb.height * inch)/2
	else:
		args.posxoff=args.posyoff=0

	display=pygame.display.set_mode(winsize(pcb,args.twosided))
	display.fill(WHITE)
	pygame.display.set_caption('Millefori')
	pygame.event.clear()
	showbuttons()
	showhelp()
	pcb.pos=(0,0)
	while True:
		showscreen(pcb,currentselection)
		event=pygame.event.wait()
		updatepos(pcb,event)
		commonprocess(pcb,event,args)
		if (mode == EDIT):
			editprocess(pcb,event)
		elif (mode == SELECT):
			selectprocess(pcb,event)

def updatepos(pcb,event):
	if event.type in (MOUSEMOTION,MOUSEBUTTONUP,MOUSEBUTTONDOWN): pcb.pos=event.pos

def setedit(pcb):
	global mode
	mode=EDIT
	drawbutton(buttons[0],1)
	drawbutton(buttons[1])
	showhelp()
	currentselection=None
	pass

def setselect(pcb):
	global mode
	mode=SELECT
	drawbutton(buttons[1],1)
	drawbutton(buttons[0])
	showhelp()
	currentselection=None
	pass

def undoadd(pcb):
	undostack.append(copy.deepcopy(pcb.__dict__))
	if len(undostack) > 20:
		undostack.pop(0)

def undo(pcb):
	if len(undostack) > 0:
		pos=pcb.pos
		pcb.__dict__=undostack.pop()
		pcb.pos=pos

def save(pcb):
	file_save(pcb,pcb.filename)
	print pcb.filename, "saved"
	pass

def terminate(pcb):
	pygame.quit()
	sys.exit()

helppos=(5,180)
helpsize=(xoff-5,16)
helpmsg=[["click/drag:"," =extend"," +shift=new track"," +ctrl=add/del hole","ctrl-z=undo"],
	["click/drag=select","ctrl-c=copy","ctrl-x=cut","ctrl-v=paste","ctrl-r=rot.clipboard"]]

def showhelp():
	posx,posy=helppos
	sizex,sizey=helpsize
	for i in range(len(helpmsg[mode])):
		thisy=posy + i*sizey
		r=Rect(posx,thisy,sizex,sizey)
		pygame.draw.rect(display,WHITE,r)
		msg = helpmsg[mode][i]
		font = pygame.font.Font(None, 14)
		textImg = font.render(msg, 1, BLACK)
		display.blit(textImg, (posx,thisy))

pospos=(5,-20)
possize=(xoff-5,16)

def showpos(pcb,args):
	x,y=pcb.pos
	i,j,rear=mousecoord(pcb)
	posx,posy=pospos
	winx,winy=winsize(pcb,False)
	x = winx - x - 10
	if y > (10 * pcb.height + 10):
		y = winy + (10 * pcb.height) - y
		color=RED
	else:
		y = winy - y - 10
		color=BLACK
	posy+=winy
	if pcb.rear: posy += 10 * pcb.height + 10
	sizex,sizey=possize
	r=Rect(posx,posy,sizex,sizey)
	pygame.draw.rect(display,WHITE,r)
	msg=''
	if i>0:
		msg+="{},{}[{}]".format(i,j,pcb.table[j][i],i,j)
	if rear:
		if x>=0 and y>=0 and x<=pcb.width*10 and y<=pcb.height*10:
			xx,yy=x * inch / 10 + args.posxoff, y * inch /10 + args.posyoff
#xx,yy=x * inch / 10, y * inch /10
			if args.xyflip:
				xx,yy=yy,xx
			elif not args.noxflip:
				xsize,_=pcb.size
				xx=xsize - xx
			msg+="X{:.2f}Y{:.2f}".format(xx, yy)
	else:
		if x>=0 and y>=0 and x<=pcb.width*10 and y<=pcb.height*10:
			xx,yy=x * inch / 10 + args.posxoff, y * inch /10 + args.posyoff
#xx,yy=x * inch / 10, y * inch /10
			msg+="X{:.2f}Y{:.2f}".format(xx,yy)
	if msg:
		font = pygame.font.Font(None, 14)
		textImg = font.render(msg,1,color)
		display.blit(textImg, (posx,posy))
		
buttons=[("edit",(10,5,80,25),setedit),
				("select",(10,35,80,25),setselect),
				("undo",(10,65,80,25),undo),
				("save",(10,100,80,25),save),
				("quit",(10,130,80,25),terminate)]

def drawbutton(button,highlight=0):
	global display
	label,pos,fun=button
	r=Rect(pos)
	if highlight:
		pygame.draw.rect(display,HIGHLIGHT,r)
	else:
		pygame.draw.rect(display,WHITE,r)
	pygame.draw.rect(display,BLACK,r,2)
	font = pygame.font.Font(None, 20)
	textImg = font.render(label, 1, BLACK)
	textpos = textImg.get_rect()
	textpos.centerx = r.centerx
	textpos.centery = r.centery
	display.blit(textImg, textpos)

def getfun(m):
	for l,p,f in buttons:
		if Rect(p).collidepoint(m):
			return f
	return None

def singlesided(pcb):
	display=pygame.display.set_mode(winsize(pcb,False))
	display.fill(WHITE)
	pygame.display.set_caption('Millefori')
	pygame.event.clear()
	showbuttons()
	showhelp()
	redrawscreen()
	showscreen(pcb,currentselection)
	undoadd(pcb)
	pcb.rear=[]

def twosided(pcb):
	display=pygame.display.set_mode(winsize(pcb,True))
	display.fill(WHITE)
	pygame.display.set_caption('Millefori')
	pygame.event.clear()
	showbuttons()
	showhelp()
	redrawscreen()
	showscreen(pcb,currentselection)
	undoadd(pcb)
	pcb.rear=[[1 if pcb.valid(i,j) else 0 for i in range(pcb.width+2)]
	        for j in range(pcb.height+2)]
	for i in range(1,pcb.width+1):
		for j in range(pcb.height+1):
			if pcb.drill[j][i]: pcb.rear[j][i]=pcb.table[j][i]

def commonprocess(pcb,event,args):
	if event.type == QUIT:
		terminate(pcb)
	elif event.type == KEYDOWN:
		mods=pygame.key.get_mods()
		if mods & KMOD_CTRL:
			if event.key == K_q:
				terminate(pcb)
			if event.key == K_s:
				save(pcb)
			if event.key == K_z:
				undo(pcb)
			if event.key == K_1:
				singlesided(pcb)
			if event.key == K_2:
				twosided(pcb)
	elif event.type == MOUSEBUTTONDOWN:
		x,y=event.pos
		if x<xoff:
			f = getfun((x,y))
			if f:
				e2=pygame.event.wait()
				if e2.type == MOUSEBUTTONUP:
					f2 = getfun(event.pos)
					if f2 == f:
						f(pcb)
					pcb.pos=event.pos
	showpos(pcb,args)
	
def showbuttons():
	for b in buttons:
		drawbutton(b)
	drawbutton(buttons[0],1)

def tableinit():
	for i in range(height+2):
		table.append([])
		drill.append([])
		for j in range(width+2):
			table[i].append(1)
			drill[i].append(0)
	for i in range(width+2):
		table[0][i]=table[height+1][i]=0
	for j in range(height+2):
		table[j][0]=table[j][width+1]=0

class screenstatus_class:
	hline={}
	vline={}
	ccolor={}
	cdrill={}
screenstatus=screenstatus_class()

def redrawscreen():
	screenstatus.hline={}
	screenstatus.vline={}
	screenstatus.ccolor={}
	screenstatus.cdrill={}

def showscreen(pcb,select=None):
	if pcb.rear:
		table = pcb.table+pcb.rear[1:]
		drill = pcb.drill+pcb.drill[1:]
	else:
		table=pcb.table
		drill=pcb.drill
	ii,jj,_=mousecoord(pcb,ext=True)
	if mode == EDIT:
		hi=table[jj][ii] if ii>0 else -1
	else:
		hi = -1
# hline
	for i in range(pcb.width):
		for j in range(len(table)-1):
			if (table[j][i+1] == table[j+1][i+1]):
				color=GRAY
			else:
			 	color=BLACK
			if screenstatus.hline.get((i,j),None) != color:
				pygame.draw.line(display,color,(i*10+11+xoff,j*10+10),(i*10+19+xoff,j*10+10))
				screenstatus.hline[i,j]=color
# vline
	for i in range(pcb.width+1):
		for j in range(len(table)-2):
			if (table[j+1][i] == table[j+1][i+1]):
				color=GRAY
			else:
				color=BLACK
			if screenstatus.vline.get((i,j),None) != color:
				pygame.draw.line(display,color,(i*10+10+xoff,j*10+11),(i*10+10+xoff,j*10+19))
				screenstatus.vline[i,j]=color
# select/highlight
	h1=pcb.height+1
	for i in range(1,pcb.width+1):
		for j in range(1,len(table)-1):
			color=WHITE	
			if table[j][i]==hi:
				color=GREEN
			elif j+h1 < len(table)-1 and table[j+h1][i]==hi:
				color=LGREEN
			elif j-h1 > 0 and table[j-h1][i]==hi:
				color=LGREEN
			if mode == SELECT and select:
				mini,minj,maxi,maxj=select
				if i >= mini and j >= minj and i <= maxi and j <= maxj:
					color=RED
				if i >= mini and j >= minj+pcb.height+1 and i <= maxi and j <= maxj+pcb.height+1:
					color=RED

			if i==ii:
				color=[c*11//12 for c in color]
			if (jj-j)%(pcb.height+1) == 0:
				color=[c*11//12 for c in color]
			if screenstatus.ccolor.get((i,j),None) != color or drill[j][i] != screenstatus.cdrill.get((i,j),None):
				pygame.draw.rect(display,color,Rect(i*10+1+xoff,j*10+1,9,9))
				screenstatus.ccolor[i,j]=color
				screenstatus.cdrill[i,j]=0
# drill
	for i in range(1,pcb.width+1):
		for j in range(1,len(drill)-1):
			if drill[j][i] != screenstatus.cdrill.get((i,j),None):
				if drill[j][i]:
					pygame.draw.line(display,BLACK,(i*10+4+xoff,j*10+4),(i*10+6+xoff,j*10+6))
					pygame.draw.line(display,BLACK,(i*10+6+xoff,j*10+4),(i*10+4+xoff,j*10+6))
				screenstatus.cdrill[i,j]=drill[j][i]
	pygame.display.flip()

millekey="Th1s1sa1000foricl1pb0ard"
def loadXclipboard():
	try:
		p = subprocess.Popen(['xclip', '-o'], stdout=subprocess.PIPE)	
		stdout, stderr = p.communicate()
		stdout=stdout.replace("'",'"')
		loaded=json.loads(stdout)
		if not set(loaded.keys())=={millekey,'table','drill','rear'}:
			raise IOError
		if loaded[millekey] != millekey:
			raise IOError
		for i in 'table','drill','rear':
			clipboard.__dict__[i]=loaded[i]
	except:
		pass

def storeXclipboard():
	clipdict={k: clipboard.__dict__[k] for k in ('table','drill','rear')}
	clipdict[millekey]=millekey
	jsonclip=bytes(json.dumps(clipdict))
	try:
		p = subprocess.Popen(['xclip', '-selection', 'c'], stdin=subprocess.PIPE)	
		p.communicate(input=jsonclip)
		p = subprocess.Popen(['xclip'], stdin=subprocess.PIPE)	
		p.communicate(input=jsonclip)
	except:
		pass

def table2clipboard(pcb):
	if currentselection:
		mini,minj,maxi,maxj=currentselection
		clipboard.table=[]
		clipboard.drill=[]
		clipboard.rear=[]
		for i in range(mini,maxi+1):
			clipboard.table.append([])
			clipboard.drill.append([])
			for j in range(minj,maxj+1):
				clipboard.table[i-mini].append(pcb.table[j][i])
				clipboard.drill[i-mini].append(pcb.drill[j][i])
		if pcb.rear:
			for i in range(mini,maxi+1):
				clipboard.rear.append([])
				for j in range(minj,maxj+1):
					clipboard.rear[i-mini].append(pcb.rear[j][i])

def rotateclipboard():
	if len(clipboard.table) > 0:
		newtable=[]
		newdrill=[]
		flip=len(clipboard.table)-1;
		for j in range(len(clipboard.table[0])):
			newtable.append([])
			newdrill.append([])
		for i in range(len(clipboard.table)):
			for j in range(len(clipboard.table[i])):
				newtable[j].append(clipboard.table[flip-i][j])
				newdrill[j].append(clipboard.drill[flip-i][j])
		if clipboard.rear:
			newrear=[]
			for j in range(len(clipboard.table[0])):
				newrear.append([])
			for i in range(len(clipboard.table)):
				for j in range(len(clipboard.table[i])):
					newrear[j].append(clipboard.rear[flip-i][j])
			clipboard.rear=newrear
		clipboard.table=newtable
		clipboard.drill=newdrill

def clipboard2table(pcb):
	global currentselection
	fi,fj,_=mousecoord(pcb)
	if fi>0:
		maxi,maxj=fi,fj
		undoadd(pcb)
		for i in range(len(clipboard.table)):
			for j in range(len(clipboard.table[i])):
				if i+fi <= pcb.width and j+fj <= pcb.height:
					maxi,maxj=i+fi,j+fj
					pcb.table[j+fj][i+fi]=clipboard.table[i][j]
					pcb.drill[j+fj][i+fi]=clipboard.drill[i][j]
					if pcb.rear:
						if clipboard.rear:
							pcb.rear[j+fj][i+fi]=clipboard.rear[i][j]
						else:
							if clipboard.drill[i][j]: pcb.rear[j+fj][i+fi]=clipboard.table[i][j]
		currentselection=fi,fj,maxi,maxj

def clipboard2table1(pcb):
	global currentselection
	fi,fj,rear=mousecoord(pcb)
	if fi>0:
		undoadd(pcb)
		maxi,maxj=fi,fj
		for i in range(len(clipboard.table)):
			for j in range(len(clipboard.table[i])):
				if i+fi <= pcb.width and j+fj <= pcb.height:
					maxi,maxj=i+fi,j+fj
					if rear:
						if clipboard.rear:
							pcb.rear[j+fj][i+fi]=clipboard.rear[i][j]
							pcb.drill[j+fj][i+fi]=clipboard.drill[i][j]
							if clipboard.drill[i][j]: pcb.table[j+fj][i+fi]=clipboard.rear[i][j]
					else:
						pcb.table[j+fj][i+fi]=clipboard.table[i][j]
						pcb.drill[j+fj][i+fi]=clipboard.drill[i][j]
						if pcb.rear and clipboard.drill[i][j]:
							pcb.rear[j+fj][i+fi]=clipboard.table[i][j]
		currentselection=fi,fj,maxi,maxj

def eraseselection(pcb):
	if currentselection:
		undoadd(pcb)
		mini,minj,maxi,maxj=currentselection
		for i in range(mini,maxi+1):
			for j in range(minj,maxj+1):
				pcb.table[j][i]=1
				pcb.drill[j][i]=0
				if pcb.rear:
					pcb.rear[j][i]=1

def renumberselection(pcb):
	colormap={}
	if currentselection:
		undoadd(pcb)
		mini,minj,maxi,maxj=currentselection
		for i in range(mini,maxi+1):
			for j in range(minj,maxj+1):
				color=pcb.table[j][i]
				colormap[color]=colormap.get(color,[])+[(0,i,j)]
				if pcb.rear:
					color=pcb.rear[j][i]
					colormap[color]=colormap.get(color,[])+[(1,i,j)]
		for c in colormap:
			if c > 1:
				newc=new_track(pcb)
				for r,i,j in colormap[c]:
					if r:
						pcb.rear[j][i]=newc
					else:
						pcb.table[j][i]=newc

def selectprocess(pcb,event):
	global currentselection
	if event.type == MOUSEBUTTONDOWN:
		i,j,_=mousecoord(pcb)
		if i>0:
			while True:
				fi,fj,_=mousecoord(pcb)
				if fi>0:
					currentselection=min(i,fi),min(j,fj),max(i,fi),max(j,fj)
				if event.type == MOUSEBUTTONUP:
					break
				showscreen(pcb,currentselection)
				event=pygame.event.wait()
				updatepos(pcb,event)
		else:
				currentselection=None
	elif event.type == KEYDOWN:
		if event.key == K_ESCAPE:
			currentselection=None
		elif event.key == K_BACKSPACE:
			eraseselection(pcb)
		else:
			mods=pygame.key.get_mods()
			if mods & KMOD_CTRL:
				if event.key == K_c:
					table2clipboard(pcb)
					if xclipboard: storeXclipboard()
				elif event.key == K_v:
					if xclipboard: loadXclipboard()
					if mods & KMOD_SHIFT:
						clipboard2table1(pcb)
					else:
						clipboard2table(pcb)
				elif event.key == K_x:
					table2clipboard(pcb)
					eraseselection(pcb)
					if xclipboard: storeXclipboard()
				elif event.key == K_r:
			 		rotateclipboard()
				elif event.key == K_n:
			 		renumberselection(pcb)

def mousecoord(pcb,ext=False):
	x,y=pcb.pos
	i,j=(x-xoff)//10,y//10
	if pcb.valid(i,j):
		return (i,j,False)
	elif pcb.rear and pcb.valid(i,pcb.j(j)):
		return (i,j,True) if ext else (i,pcb.j(j),True)
	else:
		return (0,0,False)

def new_track(pcb):
	d={}
	for i in range(pcb.width+1):
		for j in range(pcb.height+1):
			d[pcb.table[j][i]]=d.get(pcb.table[j][i],0)+1
	count=2
	while d.get(count,0) > 0:
		count += 1
	return count

def flood(pcb,newc):
		fi,fj,rear=mousecoord(pcb)
		c=pcb.rear[fj][fi] if rear else pcb.table[fj][fi]
		if not c == newc:
			q=[(fi,fj,rear)]
			while q:
				i,j,r=q.pop()
				if pcb.valid(i,j) and c == (pcb.rear[j][i] if r else pcb.table[j][i]):
					if r:
						oldc,pcb.rear[j][i]=pcb.rear[j][i],newc
					else:
						oldc,pcb.table[j][i]=pcb.table[j][i],newc
					if newc != oldc:
						q+=[(i+1,j,r),(i-1,j,r),(i,j-1,r),(i,j+1,r)]
						if pcb.rear and pcb.drill[j][i]:
							pcb.table[j][i]=pcb.rear[j][i]=newc
							q+=[(i+1,j,not r),(i-1,j,not r),(i,j-1,not r),(i,j+1,not r)]

def editprocess(pcb,event):
	if event.type == MOUSEBUTTONDOWN:
		i,j,rear=mousecoord(pcb)
		if event.button==3 and i>0:
			undoadd(pcb)
			newc=pcb.rear[j][i] if rear else pcb.table[j][i]
			staticpcb=copy.deepcopy(pcb)
			while True:
				pcb.drill=copy.deepcopy(staticpcb.drill)
				pcb.rear=copy.deepcopy(staticpcb.rear)
				pcb.table=copy.deepcopy(staticpcb.table)
				flood(pcb,newc)
				fi,fj,_=mousecoord(pcb)
				if event.type == MOUSEBUTTONUP:
					break;
				showscreen(pcb)
				event=pygame.event.wait()
				updatepos(pcb,event)
		elif event.button==1 and i>0:
			mods=pygame.key.get_mods()
			if mods & KMOD_CTRL:
				undoadd(pcb)
				oldd=pcb.drill
				staticpcb=copy.deepcopy(pcb)
				while True:
					pcb.drill=copy.deepcopy(staticpcb.drill)
	 				pcb.rear=copy.deepcopy(staticpcb.rear)
	 				pcb.table=copy.deepcopy(staticpcb.table)
					fi,fj,_=mousecoord(pcb)
					if fi>0:
						for ii in range(min(i,fi),max(i,fi)+1):
							for jj in range(min(j,fj),max(j,fj)+1):
								pcb.drill[jj][ii]=1 - oldd[j][i]
								if rear:
									pcb.table[jj][ii]=pcb.rear[jj][ii]
								else:
									if pcb.rear: pcb.rear[jj][ii]=pcb.table[jj][ii]
					if event.type == MOUSEBUTTONUP:
						break;
					showscreen(pcb)
					event=pygame.event.wait()
					updatepos(pcb,event)
			else:
				if mods & KMOD_SHIFT:
					current = new_track(pcb)
				else:
					current=pcb.rear[j][i] if rear else pcb.table[j][i]
				undoadd(pcb)
				staticpcb=copy.deepcopy(pcb)
				while True:
	 				pcb.rear=copy.deepcopy(staticpcb.rear)
	 				pcb.table=copy.deepcopy(staticpcb.table)
					fi,fj,_=mousecoord(pcb)
					if fi>0:
						for ii in range(min(i,fi),max(i,fi)+1):
							for jj in range(min(j,fj),max(j,fj)+1):
								if rear:
									pcb.rear[jj][ii]=current
									if pcb.drill[jj][ii]: pcb.table[jj][ii]=current
								else:
									pcb.table[jj][ii]=current
									if pcb.rear and pcb.drill[jj][ii]: pcb.rear[jj][ii]=current
					if event.type == MOUSEBUTTONUP:
						break
					showscreen(pcb)
					event=pygame.event.wait()
					updatepos(pcb,event)

def file_save(pcb,name):
	f = open(name, "w")
	savepcb=copy.deepcopy(pcb)
	del savepcb.pos
#	json.dump(savepcb.__dict__,f,indent=2)
	json.dump(savepcb.__dict__,f)
	f.close()

def file_load(pcb):
	try:
		try:
			name=pcb.filename
			with open(name, "r") as f:
				loaded=json.load(f)
				for f in loaded:
					pcb.__dict__[f]=loaded[f]
			pcb.filename=name
			pcb.pos=(0,0)
		except TypeError:
			#old format
			with open(pcb.filename, "r") as f:
				pcb.size,pcb.table,pcb.drill=json.load(f)
			pcb.height=len(pcb.table)-2
			pcb.width=len(pcb.table[0])-2
	except IOError:
		return False
	return True
	
if __name__ == '__main__':
	main()
